﻿import type { RequestOptions } from '@@/plugin-request/request';
import type { RequestConfig } from '@umijs/max';
import { message, notification } from 'antd';
import { history } from '@umijs/max';

// 与后端约定的响应数据格式
interface ResponseStructure {
  success: boolean;
  data: any;
  code: number;
  msg: string;
  fail: boolean
}

const codeMessage: any = {
  200: '服务器成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  405: '请求方法不被允许。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。',
};

/**
 * @name 错误处理
 * pro 自带的错误处理， 可以在这里做自己的改动
 * @doc https://umijs.org/docs/max/request#配置
 */
export const errorConfig: RequestConfig = {
  // 错误处理： umi@3 的错误处理方案。
  errorConfig: {
    // 错误抛出
    // errorThrower: (res) => {
    //   const { success, data, code, msg, fail } = res as unknown as ResponseStructure;
    //   if (!success) {
    //     let error: any = new Error(msg);
    //     error.name = 'BizError';
    //     error.info = { code, msg, fail, data };
    //     throw error; // 抛出自制的错误
    //   }
    // },
    // // 错误接收及处理
    errorHandler: (error: any, opts: any) => {
      const { response } = error;
      if (response && response.status) {
        const errorText = codeMessage[(response as any).status] || response.statusText;
        const { status, path } = response;
        notification.error({
          message: `请求错误 ${status}: ${path}`,
          description: errorText,
        });
      }

      if (!response) {
        notification.error({
          description: '您的网络发生异常，无法连接服务器',
          message: '网络异常',
        });
      }
      throw error;
    },
  },

  // 请求拦截器
  requestInterceptors: [
    (config: RequestOptions) => {
      // 拦截请求配置，进行个性化处理。
      const url = config?.url;
      if (config?.headers) {
        config.headers['token'] = localStorage.getItem('Admin-Token') || ''
      }
      return { ...config, url };
    },
  ],

  // 响应拦截器
  responseInterceptors: [
    (response) => {
      // 拦截响应数据，进行个性化处理
      const { data } = response as unknown as ResponseStructure;
      if (data?.success === false) {
        let msg: any
        switch (data?.code) {
          case 500:
            msg = sessionStorage.getItem('msg')
            if (data.msg === '令牌验证失败') {//
              localStorage.removeItem('Admin-Token')
              if (!msg) {
                sessionStorage.setItem('msg', 'true')
                message.error({
                  content: data?.msg,
                  onClose: () => {
                    localStorage.removeItem('Admin-Token')
                    history.push('/user/login')
                    sessionStorage.removeItem('msg')
                  }
                });
              }
            }
            if (!msg) {
              sessionStorage.setItem('msg', 'true')
              message.error({
                content: data.msg,
                onClose: () => {
                  sessionStorage.removeItem('msg')
                }
              });
            }
            break
          case 310:
            msg = sessionStorage.getItem('msg')
            localStorage.removeItem('Admin-Token')
            history.push('/user/login')
            if (!msg) {
              sessionStorage.setItem('msg', 'true')
              notification.error({
                message: data?.msg,
                onClose: () => {
                  sessionStorage.removeItem('msg')
                }
              });
            }
            break
          default:
            message.error(data?.msg || '请求失败！');
        }
      }
      return response;
    },
  ],
};
